"""
	Launch VAMOS 2016 experiment utility
	
	author : Xavier Devroey <xavier.devroey@unamur.be>
"""

import sys
import os
import re
import csv
import subprocess
import time


def usage():
	print "usage: lauchExp.py <model.fts> <fd.dimacs> <nbFaultyStates> <nbFaultyActions>  <nbFaultyTransitions>"




####################
# Main program
####################


if len(sys.argv) < 6:
	print "Wrong number of arguments!"
	usage()
	exit()


ftsFile = sys.argv[1]
dimacsFile = sys.argv[2]
nbFaultyStates = "{}".format(int(sys.argv[3]))
nbFaultyActions = "{}".format(int(sys.argv[4]))
nbFaultyTransitions = "{}".format(int(sys.argv[5]))

dissimilarTimeMult = [1, 2, 10, 100]

allActionsExt = "allactions.testset"
allActionsJar = "splc15-allactions-selection-0.0.1-SNAPSHOT-jar-with-dependencies.jar"

randomExt = "random.testset"
randomJar = "random-selection-0.0.1-SNAPSHOT-jar-with-dependencies.jar"

dissimilarTestSetsDir = "dissimilar-testsets-{}/"
dissimilarJar = "dissimilar-selection-0.0.1-SNAPSHOT-jar-with-dependencies.jar"

faultsInjectExt = "faults.csv"
faultsInjectJar = "fault-injection-0.0.1-SNAPSHOT-jar-with-dependencies.jar"

statsExt = "log.txt"


with open("{}.{}".format(ftsFile, statsExt), 'w') as log:

	# Launch all-actions generation
	print "Launching All actions test case generation"
	startingTime = time.time()
	allactionsTestFile = "{}.{}".format(ftsFile,allActionsExt)
	subprocess.call(["java", "-Xmx2048m", "-Xms64m", "-jar", allActionsJar, ftsFile, dimacsFile, allactionsTestFile])
	endingTime = time.time()
	duration = int(max(endingTime - startingTime, 1.00) * 1000)
	f = open("{}.{}".format(ftsFile,allActionsExt), 'r')
	cpt = 0
	for line in f:
		cpt = cpt + line.count("<abstract-test-case id")
	nbrTestCases = "{}".format(cpt)
	log.write("allstatestime={}\n".format(endingTime - startingTime))
	log.write("allstatesnbrtests={}\n".format(nbrTestCases))
	
	# Launch random generation
	print "Launching random test case generation"
	startingTime = time.time()
	randomTestFile = "{}.{}".format(ftsFile,randomExt)
	subprocess.call(["java", "-Xmx2048m", "-Xms64m", "-jar", randomJar, ftsFile, dimacsFile, nbrTestCases, randomTestFile])
	endingTime = time.time()
	log.write("randomtime={}\n".format(endingTime - startingTime))

	for i in dissimilarTimeMult:

		# Launch dissimilar generation
		print "Launching dissimilar generations {}".format(i)
		try:
			os.mkdir(dissimilarTestSetsDir.format(i))
		except OSError as e:
			pass
		startingTime = time.time()
		subprocess.call(["java", "-Xmx2048m", "-Xms64m", "-jar", dissimilarJar, ftsFile, dimacsFile, nbrTestCases, "{}".format(duration * i), dissimilarTestSetsDir.format(i)])
		endingTime = time.time()
		log.write("dissimilartotaltime={}\n".format(endingTime - startingTime))

	# All states fault injection
	print "All actions fault injection and execution"
	startingTime = time.time()
	subprocess.call(["java", "-Xmx2048m", "-Xms64m", "-jar", faultsInjectJar, ftsFile, dimacsFile, allactionsTestFile, nbFaultyStates, nbFaultyActions, nbFaultyTransitions, "{}.{}".format(allactionsTestFile, faultsInjectExt)])
	endingTime = time.time()
	log.write("allstatesfaultsinjection={}\n".format(endingTime - startingTime))

	# Random fault injection
	print "Random fault injection and execution"
	startingTime = time.time()
	subprocess.call(["java", "-Xmx2048m", "-Xms64m", "-jar", faultsInjectJar, ftsFile, dimacsFile, randomTestFile, nbFaultyStates, nbFaultyActions, nbFaultyTransitions, "{}.{}".format(randomTestFile, faultsInjectExt)])
	endingTime = time.time()
	log.write("randomfaultsinjection={}\n".format(endingTime - startingTime))


	for i in dissimilarTimeMult:

		# Dissimilar fault injection
		for testsetFile in [ f for f in os.listdir(dissimilarTestSetsDir.format(i)) if f.endswith(".testset") ]:
			print "Dissimilar fault injection and execution {}/{}".format(dissimilarTestSetsDir.format(i), testsetFile)
			startingTime = time.time()
			dissimilarTestFile = "{}/{}".format(dissimilarTestSetsDir.format(i), testsetFile)
			print dissimilarTestFile
			subprocess.call(["java", "-Xmx2048m", "-Xms64m", "-jar", faultsInjectJar, ftsFile, dimacsFile, dissimilarTestFile, nbFaultyStates, nbFaultyActions, nbFaultyTransitions, "{}/{}.{}".format(dissimilarTestSetsDir.format(i), testsetFile, faultsInjectExt)])
			endingTime = time.time()
			log.write("dissimilarfaultsinjection{}{}={}\n".format(i, testsetFile.replace('-', '').replace('.testset',''), endingTime - startingTime))


	log.close()



